/* 
 * Unix System Interface
 * Nils M Holm, 2024
 * In the public domain / 0BSD license
 */

#include <string.h>
#include <unistd.h>
#include <dirent.h>
#include <time.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/select.h>

#define byte	unsigned char
#define cell	long

cell t3x_mkdir(char *s) { return mkdir(s, 0755); }

cell t3x_rmdir(char *s) { return rmdir(s); }

cell t3x_chdir(char *s) { return chdir(s); }

cell t3x_getdir(cell k, char *b) {
	getcwd(b, k);
	return strlen(b);
}

cell t3x_dup(cell fd) { return dup(fd); }

cell t3x_dup2(cell fd2, cell fd) { return dup2(fd, fd2); }

DIR *dir;

cell t3x_opendir(void) {
	dir = opendir(".");
	return NULL == dir? -1: 0;
}

cell t3x_readdir(cell k, char *b) {
	struct dirent	*d;
	int		n;

	d = readdir(dir);
	if (NULL == d) return 0;
	n = strlen(d->d_name);
	strncpy(b, d->d_name, k);
	b[k-1] = 0;
	return strlen(b);
}

cell t3x_closedir(void) {
	closedir(dir);
	return 0;
}

cell t3x_isdir(char *s) {
	struct stat	sb;

	if (stat(s, &sb) < 0) return 0;
	return sb.st_mode & S_IFDIR? -1: 0;
}

enum {	T_CENT = 0, T_YEAR, T_MONTH, T_DAY,
 	T_HOUR,     T_MIN,  T_SEC,   T_HSEC };

static int maketime(time_t *tm, byte *tb) {
	struct	tm	*lt;

	lt = localtime(tm);
	if (NULL == lt) return -1;
	tb[T_CENT]  = (lt->tm_year + 1900) / 100;
	tb[T_YEAR]  = (lt->tm_year + 1900) % 100;
	tb[T_MONTH] = lt->tm_mon;
	tb[T_DAY]   = lt->tm_mday;
	tb[T_HOUR]  = lt->tm_hour;
	tb[T_MIN]   = lt->tm_min;
	tb[T_SEC]   = lt->tm_sec;
	tb[T_HSEC]  = 0;
	return 0;
}

cell t3x_gettime(byte *tb) {
	time_t	tm;

	time(&tm);
	return maketime(&tm, tb);
}

cell t3x_getftime(byte *tb, char *s) {
	struct stat	sb;

	if (stat(s, &sb) < 0) return -1;
#ifdef __APPLE__
	return maketime(&sb.st_mtimespec.tv_sec, tb);
#else
	return maketime(&sb.st_mtim.tv_sec, tb);
#endif
}

cell t3x_getfsize(byte *sb, char *s) {
	struct stat	stb;

	if (stat(s, &stb) < 0) return -1;
	sb[0] = stb.st_size / (256*256*256);
	sb[1] = stb.st_size / (256*256);
	sb[2] = stb.st_size / 256;
	sb[3] = stb.st_size % 256;
	return 0;
}

cell t3x_spawn(char **args, char *prog) {
	char		*a[100];
	extern char	**environ;
	int		r, i;

	a[0] = prog;
	for (i=1; i<99; i++) {
		a[i] = args[i-1];
		if (0 == a[i]) break;
	}
	if (99 == i) return -1;
	r = fork();
	if (r < 0) return -1;
	if (0 == r) {
		execve(prog, a, environ);
	}
	else {
		wait(&r);
		return WEXITSTATUS(r);
	}
	return -1;
}

cell t3x_ready(cell ms, cell fd) {
	struct timeval	tv;
	fd_set		fds;

	FD_ZERO(&fds);
	FD_SET(fd, &fds);
	tv.tv_sec = 0;
	tv.tv_usec = ms * 1000;
	return select(fd+1, &fds, NULL, NULL, &tv)? -1: 0;
}
